@@ -2,12 +2,14 @@ import asyncio
 import json
 import logging
 import math
+import random
 import time
 import traceback
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set
 
 import aiohttp
 import websockets
+from websockets.exceptions import ConnectionClosedError, InvalidStatus
 
 if TYPE_CHECKING:
     from smartrent.device import Device
@@ -411,11 +413,15 @@ class Client:
                 uri = SMARTRENT_WEBSOCKET_URI.format(token)
 
                 _LOGGER.info("Connecting to Websocket...")
-                async with websockets.connect(uri) as websocket:
-                    # if we connect sucessfully at least one time, reset retries to 0
-                    retries = 0
-                    self._ws = websocket
-
+                headers = {"Authorization": f"Bearer {token}"}
+                async with websockets.connect(
+                    uri,
+                    additional_headers=headers,
+                    ping_interval=15,  # keep NAT and LB sessions alive
+                    ping_timeout=10,
+                    close_timeout=5,
+                    max_queue=None,  # avoid backpressure disconnects
+                ) as websocket:
                     # Join all devices to websocket connection
                     await self._async_ws_join_devices(
                         websocket, self._subscribed_devices
@@ -426,6 +432,18 @@ class Client:
                         message_list = json.loads(f"{message}")
                         formatted_resp = message_list[4]
                         device_id = message_list[2].split(":")[-1]
+                        # If server indicates channel or auth error,
+                        # refresh token and reconnect
+                        if (
+                            message_list[3] in ("phx_error", "phx_close")
+                            or "unauthoriz" in json.dumps(formatted_resp).lower()
+                        ):
+                            _LOGGER.warning(
+                                "WS reported auth or channel error,"
+                                " refreshing token and reconnecting"
+                            )
+                            await self._async_refresh_token()
+                            break
 
                         event_type = formatted_resp.get("type", "")
                         event_name = formatted_resp.get("name", "")
@@ -446,7 +464,9 @@ class Client:
                                     await device._update(formatted_resp)
                         else:
                             _LOGGER.info(str(message))
-
+            except (ConnectionClosedError, ConnectionResetError) as exc:
+                _LOGGER.warning("WebSocket closed by peer: %s", exc)
+                self._ws = None
             except Exception as exc:
                 _LOGGER.warning(
                     "Exception occured! %s %s", type(exc).__name__, type(exc)
@@ -456,7 +476,7 @@ class Client:
                 # set websocket to None
                 self._ws = None
 
-                wait_time = 1.25**retries
+                wait_time = 1.25**retries + random.uniform(0, 0.5)  # jitter
                 wait_time = wait_time if wait_time < 300 else 300
                 _LOGGER.warning("Retrying websocket in %s seconds...", wait_time)
 
@@ -477,19 +497,22 @@ class Client:
         payload = COMMAND_PAYLOAD.format(
             attribute_name=attribute_name, value=value, device_id=device._device_id
         )
-        try:
-            await self._async_send_payload(device, payload)
-        except websockets.exceptions.InvalidStatusCode as exc:  # type: ignore
-            _LOGGER.debug(
-                'Possible issue during send_payload: "%s" '
-                "Refreshing token and retrying",
-                exc,
-            )
-
-            # update token once
-            await self._async_refresh_token()
-
-            await self._async_send_payload(device, payload)
+        for attempt in range(2):  # Try twice
+            try:
+                await self._async_send_payload(device, payload)
+                return  # Success, exit
+            except InvalidStatus as exc:
+                if attempt == 0:  # First attempt failed
+                    _LOGGER.debug(
+                        'Possible issue during send_payload: "%s" '
+                        "Refreshing token and retrying",
+                        exc,
+                    )
+                    # update token once
+                    await self._async_refresh_token()
+                else:
+                    # Second attempt failed, re-raise
+                    raise
 
     async def _async_send_payload(self, device: "Device", payload: str):
         """
@@ -499,12 +522,19 @@ class Client:
 
         ``payload`` string of device attributes
 
-        Throws ``websockets.exceptions.InvalidStatusCode`` upon bad websocket event
+        Throws ``websockets.exceptions.InvalidStatus`` upon bad websocket event
         """
         _LOGGER.info("sending payload %s", payload)
 
         uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
+        headers = {"Authorization": f"Bearer {self._token}"}
 
-        async with websockets.connect(uri) as websocket:  # type: ignore
+        async with websockets.connect(
+            uri,
+            additional_headers=headers,
+            ping_interval=15,
+            ping_timeout=10,
+            close_timeout=5,
+        ) as websocket:  # type: ignore
             await self._async_ws_joiner(websocket, device)
             await websocket.send(payload)
